(function () {

    "use strict";

    angular.module("enrMapModule")
           .directive("enrMap", ["mapSelectedRegion", function (mapSelectedRegion) {

               // When the directive is assigned new geo or election data, redraw the map.
               function link(scope) {
                   scope.$watch("mapGeo", function () {
                       if (scope.mapGeo !== undefined) {
                           mapSelectedRegion.clearSelectedRegion();
                           scope.zoomOut();
                       }
                   });
               }

               return {
                   scope: { mapGeo: "=", mapColors: "=", electionData: "=", referendumsMap: "=" },
                   controller: "enrMapController",
                   templateUrl: "Scripts/EnrAngular/EnrMap/enrMap.html",
                   link: link,
                   restrict: "E"
               }
           }])
           .controller("enrMapController", ["$scope", "$filter", "topoJsonService", "d3Service", "mapSvgPathBuilder", "mapClassGenerator", "mapRegionLeaderCalculator",
                                            "$window", "enrModeManager", "jsonDataFetcher", "mapFillColorCalculator", "mapSelectedRegion", "detailsData", "regionPropertiesParser",
                                            function ($scope, $filter, topoJsonService, d3Service, mapSvgPathBuilder, mapClassGenerator, mapRegionLeaderCalculator, $window,
                                                      enrModeManager, jsonDataFetcher, mapFillColorCalculator, mapSelectedRegion, detailsData, regionPropertiesParser) {

               // Offices/Referendums mode handling
               $scope.$on(enrModeManager.modeChangeEventName, function(event, newMode) {
                   if (newMode === enrModeManager.modeEnum.referendums) {
                       jsonDataFetcher.fetchMultipleJson([$scope.referendumsMap, "statewideElectionsR"], [false, true], [true, false], function (jsonResults) {
                           var mapJson = jsonResults[0];
                           var electionData = jsonResults[1];
                                   
                           $scope.electionData = electionData;
                           $scope.mapGeo = mapJson;
                       });
                   }
                   else {
                       // enrOfficeSelection Directive Handles This (since it knows what item is currently picked, which will dictate the map data assigned)
                   }
               });

               // Region Drawing
               $scope.generateRegionClass = function (geometryObjProperties) {
                   var regionId = regionPropertiesParser.getRegionPrimaryId(geometryObjProperties);
                   var isRegionSelected = mapSelectedRegion.isRegionSelected(geometryObjProperties);

                   return mapClassGenerator.generateRegionClass(regionId, isRegionSelected);
               };
               $scope.regionFillColor = function (geometryObjProperties) {
                   var fillColor = "#000000";

                   if (enrModeManager.isReferendumsMode())
                       fillColor = mapFillColorCalculator.calculateFillColorReferendums(geometryObjProperties, $scope.electionData, $scope.mapColors);

                   else if (enrModeManager.isOfficeMode())
                       fillColor = mapFillColorCalculator.calculateFillColorOffice(geometryObjProperties, $scope.electionData, $scope.mapColors);

                   return fillColor;
               };

               // Tooltips
               $scope.getRegionDisplayName = function (geometryObjProperties) {
                   return regionPropertiesParser.getRegionDisplayName(geometryObjProperties);
               }
               $scope.regionMouseOver = function (geometryObjProperties) {
                   $(".map-region." + regionPropertiesParser.getRegionPrimaryId(geometryObjProperties)).tooltip("show");
               };
               $scope.regionMouseLeave = function (geometryObjProperties) {
                   $(".map-region." + regionPropertiesParser.getRegionPrimaryId(geometryObjProperties)).tooltip("destroy");
               };

               //Zoom Handling / Map Positioning
               var isMapZoomedIn;
               var isMouseInZoomTargetMode;
               var windowWidth = $window.innerWidth;
               var zoomInMouseX = 0;
               var zoomInMouseY = 0;
               $scope.$watch(function () {
                   if ($window.innerWidth !== windowWidth) {
                       windowWidth = $window.innerWidth;
                       mapSvgPathBuilder.injectNewPathInfoIntoMapGeoData($scope.mapGeo, windowWidth, isMapZoomedIn ? [zoomInMouseX, zoomInMouseY] : undefined);
                   }
               });
               $scope.zoomOut = function () {
                   isMapZoomedIn = false;
                   isMouseInZoomTargetMode = false;
                   $scope.zoomButtonClass = mapClassGenerator.generateZoomButtonClass(isMapZoomedIn, isMouseInZoomTargetMode, enrModeManager.isOfficeMode());;
                   $scope.mapClass = mapClassGenerator.generateMapClass(isMapZoomedIn, isMouseInZoomTargetMode);

                   mapSvgPathBuilder.injectNewPathInfoIntoMapGeoData($scope.mapGeo, windowWidth);
               };
               $scope.zoomIn = function () {
                   isMapZoomedIn = true;
                   isMouseInZoomTargetMode = false;
                   $scope.zoomButtonClass = mapClassGenerator.generateZoomButtonClass(isMapZoomedIn, isMouseInZoomTargetMode, enrModeManager.isOfficeMode());
                   $scope.mapClass = mapClassGenerator.generateMapClass(isMapZoomedIn, isMouseInZoomTargetMode);

                   mapSvgPathBuilder.injectNewPathInfoIntoMapGeoData($scope.mapGeo, windowWidth, [zoomInMouseX, zoomInMouseY]);
               };
               $scope.handleZoomClicked = function () {

                   // Zoom out happens immediately when you click the zoom out button.
                   if (isMapZoomedIn) {
                       $scope.zoomOut();
                   }

                       // Zoom in requires you to click the map to pick your zoom in spot.
                   else {
                       isMouseInZoomTargetMode = !isMouseInZoomTargetMode;

                       $scope.zoomButtonClass = mapClassGenerator.generateZoomButtonClass(isMapZoomedIn, isMouseInZoomTargetMode, enrModeManager.isOfficeMode());
                       $scope.mapClass = mapClassGenerator.generateMapClass(isMapZoomedIn, isMouseInZoomTargetMode);
                   }
               };

               // Region Selection
               $scope.handleMapRegionClicked = function (geometryObjProperties, event) {

                   // Zoom
                   if (isMouseInZoomTargetMode) {
                       zoomInMouseX = event.offsetX;
                       zoomInMouseY = event.offsetY;

                       // Firefox Support
                       if (zoomInMouseX === undefined)
                           zoomInMouseX = event.originalEvent.layerX;
                       if (zoomInMouseY === undefined)
                           zoomInMouseY = event.originalEvent.layerY;

                       $scope.zoomIn();
                   }

                   // Region Selection
                   else {
                       var clickedRegionId = regionPropertiesParser.getRegionPrimaryId(geometryObjProperties);
                       var clickedRegionElement = $(".map-region." + clickedRegionId);
                       var clickedRegionParent = clickedRegionElement.parent();
                       var isUserDeselectingRegion = clickedRegionId === mapSelectedRegion.getSelectedRegionId();

                       // Clear existing selections
                       if (mapSelectedRegion.isAnyRegionSelected()) {
                           var curSelectedRegionId = mapSelectedRegion.getSelectedRegionId();
                           mapSelectedRegion.clearSelectedRegion();
                           $(".map-region." + curSelectedRegionId).attr("class", mapClassGenerator.generateRegionClass(curSelectedRegionId, false));
                       }

                       // Make new selection (unless they deselected region by clicking on it a second time)
                       if (!isUserDeselectingRegion) {
                           mapSelectedRegion.setSelectedRegion(geometryObjProperties);
                           clickedRegionElement.attr("class", mapClassGenerator.generateRegionClass(clickedRegionId, true));
                           clickedRegionParent.append(clickedRegionElement); // Force region to be final SVG child to ensure it renders on top (so active effect is fully visible)
                       }

                       // Pull Referendums Data
                       var shouldPullRefData = enrModeManager.isReferendumsMode() && !isUserDeselectingRegion;
                       if (shouldPullRefData) {
                           var refDataFile = "JurR_" + regionPropertiesParser.getRegionPrimaryId(geometryObjProperties, false, true);

                           jsonDataFetcher.fetchSingleJson(refDataFile, true, false, function (referendumsJson) {
                               detailsData.setData(referendumsJson);
                           });
                       }

                       // Clear Referendums Data
                       if (enrModeManager.isReferendumsMode() && isUserDeselectingRegion)
                           detailsData.clearData();
                   }
               };
           }]);

}());
